## $Id: test_catalog.py 7203 2011-11-25 20:54:14Z henrik $
##
## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
import grok
import shutil
import tempfile
from zope.event import notify
from zope.catalog.interfaces import ICatalog
from zope.component import queryUtility, createObject
from zope.component.hooks import setSite
from waeup.sirp.app import University
from waeup.sirp.testing import FunctionalLayer, FunctionalTestCase
from waeup.sirp.students.student import Student
from waeup.sirp.university.faculty import Faculty
from waeup.sirp.university.department import Department

class CatalogTestSetup(FunctionalTestCase):
    # A setup for testing students catalog related stuff.
    #
    # sets up a site with some student already created.
    layer = FunctionalLayer

    def setUp(self):
        super(CatalogTestSetup, self).setUp()

        # Prepopulate ZODB
        app = University()
        self.dc_root = tempfile.mkdtemp()
        app['datacenter'].setStoragePath(self.dc_root)

        # Prepopulate the ZODB...
        self.getRootFolder()['app'] = app
        self.app = self.getRootFolder()['app']
        setSite(self.app)
        self.certificate = createObject('waeup.Certificate')
        self.certificate.code = u'CERT1'
        self.app['faculties']['fac1'] = Faculty(code=u'fac1')
        self.app['faculties']['fac1']['dep1'] = Department(code=u'dep1')
        self.app['faculties']['fac1']['dep1'].certificates.addCertificate(
            self.certificate)

        # Create student with studycourse subobject
        student = Student()
        student.fullname = u'Bob Tester'
        self.app['students'].addStudent(student)
        self.student_id = student.student_id
        self.student = self.app['students'][self.student_id]
        self.student['studycourse'].certificate = self.certificate
        # Update the catalog
        notify(grok.ObjectModifiedEvent(self.student))
        return

    def tearDown(self):
        shutil.rmtree(self.dc_root)
        super(CatalogTestSetup, self).tearDown()
        return

class StudentCatalogTests(CatalogTestSetup):

    layer = FunctionalLayer

    def test_get_catalog(self):
        # We can get an students catalog if we wish
        cat = queryUtility(ICatalog, name='students_catalog')
        assert cat is not None

    def test_search_by_id(self):
        # We can find a certain student id
        cat = queryUtility(ICatalog, name='students_catalog')
        results = cat.searchResults(student_id=(self.student_id, self.student_id))
        results = [x for x in results] # Turn results generator into list
        assert len(results) == 1
        assert results[0] is self.app['students'][self.student_id]

    def test_search_by_name(self):
        # We can find a certain name
        cat = queryUtility(ICatalog, name='students_catalog')
        results = cat.searchResults(fullname='Bob Tester')
        results = [x for x in results] # Turn results generator into list
        assert len(results) == 1
        assert results[0] is self.app['students'][self.student_id]

    def test_search_by_department(self):
        # We can find a student studying in a certain department
        cat = queryUtility(ICatalog, name='students_catalog')
        results = cat.searchResults(depcode=('dep1','dep1'))
        results = [x for x in results] # Turn results generator into list
        assert len(results) == 1
        assert results[0] is self.app['students'][self.student_id]

    def test_search_by_faculty(self):
        # We can find a student studying in a certain faculty
        cat = queryUtility(ICatalog, name='students_catalog')
        results = cat.searchResults(faccode=('fac1','fac1'))
        results = [x for x in results] # Turn results generator into list
        assert len(results) == 1
        assert results[0] is self.app['students'][self.student_id]